/*
 * tmos - delay.c
 *
 * Implementation of delay functions
 * 
 * Copyright (C) 2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/config.h>
#include <tmos/jiffies.h>
#include <tmos/watchdog.h>
#include <asm/system.h>
#include <asm/delay.h>

#if DELAY_AUTODETECT
unsigned int loops_per_jiffy = 0;
#endif

void __delay(unsigned int loops) __naked
{
#ifndef __CHECKER__
	__asm
		ar2 = 0x02
		ar3 = 0x03
		ar4 = 0x04
		ar5 = 0x05
		ar6 = 0x06
		ar7 = 0x07
		ar0 = 0x00
		ar1 = 0x01
		push	ar2
		push	ar3
		mov	r2,dpl
		mov	r3,dph
__delay_loop:
		dec	r2
		cjne	r2,#0xff,__bar
		dec	r3
__bar:
		mov	a,r2
		orl	a,r3
		jnz	__delay_loop
		pop	ar3
		pop	ar2
		ret
       __endasm;
#endif
}

#if DELAY_AUTODETECT
static unsigned int calibrate_delay(void)
{
	unsigned int lpj;
	unsigned int loopadd;
	jiffies_t ticks;
	char shorter_only = 1;

	lpj = 1 << 10;
	loopadd = 1 << 10;

	do {
		watchdog_refresh();
		ticks = get_jiffies();
		while (ticks == get_jiffies())
			;
		ticks = get_jiffies();
		__delay(lpj);
		if (get_jiffies() != ticks) {
			/* longer than 1 tick */
			loopadd >>= 1;
			lpj -= loopadd;
			shorter_only = 0;
		} else {
			if (lpj >= 65535 - loopadd) {
				lpj = 65535;
				shorter_only = 0;
			} else {
				lpj += loopadd;
			}

			if (!shorter_only)
				loopadd >>= 1;
		}
	} while (loopadd > 2);
	return lpj;	
}
#else
#define calibrate_delay()	(CONFIG_DELAY_LPJ)
#endif

#if !CONFIG_DELAY_LPJ || CONFIG_PRINT_BOGOMIPS
void delay_init(void)
{
	unsigned int lpj;

	lpj = calibrate_delay();
#if CONFIG_PRINT_BOGOMIPS
	printk("%u.%02u BogoMIPS (lpj = %u)\n",
			(unsigned int) (lpj / (1000000UL / 6 / HZ)),
			(unsigned int) (lpj / (1000000UL / 6 / 100 / HZ)) % 100,
			lpj);
#endif
#if DELAY_AUTODETECT
	loops_per_jiffy = lpj;
#endif
}
#endif
