/*
 * wysmod - lcd.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2006  Marek Piwonski <marek@podlesie.net>
 * Copyright (C) 2006..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/lcd.h>
#include <tmos/console.h>
#include <tmos/jiffies.h>
#include <asm/8051.h>
#include <asm/processor.h>

#define LCD_RS		P2_0
#define LCD_RW		P2_1
#define LCD_CLK		P2_2	/* E signal */
#define LCD_DATA	P0
#define LCD_BUSY	P0_7

#define MODE_WAIT	0x01
#define MODE_DATA	0x02

#define	D_CLEAR	 		0x01
#define	HOME 		 	0x02
#define	ENTRY_MOD		0x06
#define	D_CONTROL		0x0c
#define	D_C_SHIFT		0x14
#define	FUNCT_SET		0x38
#define	D_ADDR_SET		0x80

static void lcd_rozkaz(char dana, char mode)
{
	if (mode & MODE_WAIT) {
		LCD_CLK = 0;
		LCD_DATA = 0xff;
		LCD_RS = 0;
		LCD_RW = 1;
		LCD_CLK = 1;
		while (LCD_BUSY)
			cpu_relax();
	}
	LCD_CLK = 0;
	LCD_DATA = dana;
	LCD_RS = 0;
	if (mode & MODE_DATA)
		LCD_RS = 1;
	LCD_RW = 0;
	LCD_CLK = 1;
	LCD_CLK = 0;
}

void console_init(void)
{
	lcd_rozkaz(0x30, 0);
	usleep(2000);
	lcd_rozkaz(0x30, 0);
	lcd_rozkaz(FUNCT_SET, MODE_WAIT);
	lcd_rozkaz(ENTRY_MOD, MODE_WAIT);
	lcd_rozkaz(D_CONTROL, MODE_WAIT);
	lcd_rozkaz(D_CLEAR, MODE_WAIT);
}

void c_putchar_p(char pos, char c)
{
	if (pos >= 20)
		pos += 44;
	pos |= D_ADDR_SET; 
	lcd_rozkaz(pos, MODE_WAIT);
	lcd_rozkaz(c, MODE_DATA | MODE_WAIT);
}

device_initcall(console_init)
